<?php


/**
 * @file
 * Bubbletimer main page
 */


/**
 * Bubbletimer interface
 * Render
 * 	- title
 * 	- date selector form part
 * 	- activity form
 * @return string
 */
function bubbletimer_page() {
  bubbletimer_include_js_css();
  $user = bubbletimer_get_user();
  $date = bubbletimer_get_date();
  $timestamp = strtotime(join('-', $date));

  drupal_set_title(t('Bubbletimer - @username\'s daily basis', array('@username' => $user->name)));

  $date_form = drupal_get_form('bubbletimer_date_selector_form');
  $nodes_enabled = bubbletimer_get_jobs_enable(bubbletimer_get_user());
  $node_form = drupal_get_form('bubbletimer_node_add_form', count($nodes_enabled) == 0);
  if (!empty($nodes_enabled)) {
    $bubbletimer_form = drupal_get_form('bubbletimer_activityform');
  } else {
    $bubbletimer_form = '';
    drupal_set_message('You haven\'t got any nodes assigned to bubbletimer. Select one in the form below, or visit the node\'s page.', 'warning');
  }

  return theme('bubbletimer_page', $date_form, $node_form, $bubbletimer_form);
}


/**
 * Definition of activity form
 * @return string
 */
function bubbletimer_activityform() {
  $path = drupal_get_path('module', 'bubbletimer');
  require_once $path.'/bubbletimer.slot.inc';

  $user = bubbletimer_get_user();
  $nids = bubbletimer_get_jobs_enable($user);
  $timestamp_original = bubbletimer_get_timestamp();

  $form['bt']['#theme'] = 'bt_form';

  $form['bt']['bt_wrapper'] = array(
    '#theme' => 'bt_wrapper',
  );

  // Checkboxes and titles / node / step
  foreach ((array)$nids as $nid => $node_title) {
    $timestamp = $timestamp_original;
    $form['bt']['bt_wrapper']['title'][$nid] = array(
      '#value' => '<a class="job_node" nid="'. $nid .'" href="'. url('node/'.  $nid, array('absolute' => TRUE)) .'">'.
                  check_plain((strlen($node_title) > BT_TITLE_LENGTH ? substr($node_title, 0, BT_TITLE_LENGTH) .'...' : $node_title)) .
                  ' </a><small>, <span id="bt_node_percent_'. $nid .'"></span>h</small>',
      '#nid' => $nid,
    );
    $nid_patter = bubbletimer_get_job_pattern($user->uid, $nid, date('Y-m-d', bubbletimer_get_timestamp()));
    for ($i = 0; $i < BT_STAGE_COUNT; $i++) {
      $form['bt']['bt_wrapper']['slot'][$nid][$timestamp] = array(
        '#type' => 'bubbletimer_slot',
        '#default_value' => substr($nid_patter, $i, 1) == '1' ? 1 : 0,
      );
      $timestamp += BT_SECONDS_INTERVAL;
    }
  }

  // Submit button
  $form['bt']['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => 'Save changes',
    '#suffix' => '
    	<input type="button" onclick="bubbletimer_history_backward()" value="'.t('Undo').'" />
    	<input type="button" onclick="bubbletimer_history_forward()" value="'.t('Redo').'" />
    ',
  );

  return $form;
}


/**
 * Define date selector form
 * @param array $form_state information about form
 * @return string
 */
function bubbletimer_date_selector_form($form_state) {
  $collapsed = bubbletimer_get_timestamp(BT_TIMESTAMP_ONLY_DAY) == strtotime(date('Y-m-d'));
  $form['bt_dayselector_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Day selector'),
    '#collapsible' => TRUE,
    '#collapsed' => $collapsed,
  );
  $form['bt_dayselector_fieldset']['bt_date'] = array(
    '#title' => t('Select a day you want to check'),
    '#type' => 'date',
    '#theme' => 'bubbletimer_date_field',
    '#suffix' => '
      <input type="button" id="bt_date_last" value="&laquo; '.t('Last edited day').'" />
      <input type="button" id="bt_date_prev" value="&laquo; '.t('Previous day').'" />
      <input type="button" id="bt_date_now"  value="'.t('Today').'" />
      <input type="button" id="bt_date_next" value="'.t('Next day').' &raquo;" />',
  );
  $form['bt_dayselector_fieldset']['bt_submit'] = array(
    '#type' => 'submit',
    '#value' => 'Selected day',
    '#attributes' => array('style' => 'display:none;'),
  );

  return $form;
}


/**
 * Assign or unassign a certain node to a user -> to get it into the timesheet list
 * @param integer $nid
 * @param boolean $is_assign
 */
function bubbletimer_assign_node_to_user($nid, $is_assign) {
  $user = bubbletimer_get_user();
  
  // Sanitizing
  $nid = intval($nid);
  $uid = intval($user->uid);

  $result = db_fetch_object(db_query(
    'SELECT count(*) AS count FROM {bubbletimer_node_user} WHERE uid = %d AND nid = %d;',
    $uid,
    $nid
  ));
  if ($is_assign && $result->count == 0) {
    $result = db_query('INSERT INTO {bubbletimer_node_user} (uid, nid) VALUES (%d, %d);', $uid, $nid);
    drupal_set_message(t('This node will be in your timesheet list'));
  }
  else if (!$is_assign && $result->count > 0) {
    $result = db_query('DELETE FROM {bubbletimer_node_user} WHERE uid = %d AND nid = %d;', $uid, $nid);
    drupal_set_message(t('You deleted this node from your timesheet'));
  }
  
  drupal_goto(BT_PATH);
}


/**
 * Implementation of hook_submit()
 * Saving selected day in session
 * @param array $form
 * @param arary $form_state
 */
function bubbletimer_date_selector_form_submit($form, $form_state) {
  bubbletimer_set_date($form_state['values']['bt_date']);
  drupal_set_message(t('Day is changed to: ') . date('Y. m. d. (l)', bubbletimer_get_timestamp(BT_TIMESTAMP_ONLY_DAY)));
}


/**
 * Implmentation of hook_submit()
 * Saving daily activities into database
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_activityform_submit($form, $form_state) {
  $user = bubbletimer_get_user();

  foreach ((array)$form_state['clicked_button']['#post']['slot'] as $nid => $form_row) {
    // @TODO use #tree instead of this hackish access
    $activity = '';
    foreach ((array)$form_row as $timestamp => $value) {
      $activity .= $value == '1' ? '1' : '0'; // Sanitizing
    }
    $result = db_fetch_object(db_query('
      SELECT count(*) as count FROM {bubbletimer_activity} WHERE date = \'%s\' AND nid = %d AND uid = %d;
      ', join('-', bubbletimer_get_date()), intval($nid), intval($user->uid)));
    if ($result->count > 0) {
      db_query('
        UPDATE {bubbletimer_activity} SET activity = \'%s\' WHERE date = \'%s\' AND nid = %d AND uid = %d;
        ', $activity, join('-', bubbletimer_get_date()), intval($nid), intval($user->uid));
    } else {
      db_query('
        INSERT INTO {bubbletimer_activity} (date, nid, uid, activity) VALUES (\'%s\', %d, %d, \'%s\');
        ', join('-', bubbletimer_get_date()), intval($nid), intval($user->uid), $activity);
    }

    // If node updating is enabled.
    // Ex.: Computed field needs node update to recalculate data.
    if (variable_get('bubbletimer_node_refresh', '0') == '1') {
      $node = node_load($nid);
      node_save($node);
    }
  }

  drupal_set_message(t('You saved your timesheet succesfull'));
}


/**
 * Implementation of theme_hook()
 * Just for adding a submit right next to the date selectors
 * @param array $element
 * @return string
 */
function theme_bubbletimer_date_field($element) {
  return drupal_render($element) .'<input type="submit" value="Go" />';
}


/**
 * Implementation of hook_form_alter()
 * Setup forms:
 * 	- Set activity in the form (from datas in bubbletimer table)
 * 	- Set the current date on date selector form
 * @param array $form
 * @param array $form_state
 * @param string $form_id
 */
function bubbletimer_form_alter(&$form, &$form_state, $form_id) {
  if ($form_id == 'bubbletimer_date_selector_form' && empty($_POST)) {
    $date_array = array_map(create_function('$item', 'return (int)$item;'), bubbletimer_get_date());
    $form['bt_dayselector_fieldset']['bt_date']['#value'] = $date_array;
  }
  else if ($form_id == 'bubbletimer_node_assign_form' && empty($_POST)) {
    $user = bubbletimer_get_user();
    $form['bubbletimer_node_assign_fieldset']['bt_assign_checkbox']['#value'] =
      bubbletimer_is_node_assign_to_user($user->uid, $form['bubbletimer_node_assign_fieldset']['bt_node_nid']['#value']);
  }
}


/**
 * Form for adding nodes to bt on bubbletimer's page
 * @param array $form_state
 * @param boolean $enable_nodes_empty - if no nodes enables, it gets not collapsed
 * @return array
 */
function bubbletimer_node_add_form($form_state, $enable_nodes_empty) {
  $nodes_available = bubbletimer_get_jobs_available();
  $node_types = node_get_types('types');
  foreach ((array)$node_types as $node_type) {
    $_node_type_info[$node_type->type] = $node_type->name;
  }
  foreach ((array)$nodes_available as $node_type => $nodes) {
    $node_types_available[$node_type] = $_node_type_info[$node_type];
  }

  $form['bt_nodeadd_fieldset'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => !$enable_nodes_empty,
    '#title' => t('Add new node to the timesheet'),
  );
  if (!empty($node_types_available)) {
    $form['bt_nodeadd_fieldset']['bt_node_type'] = array(
      '#type' => 'select',
      '#title' => t('Node type'),
      '#options' => $node_types_available,
    );
  } else {
    $form['bt_nodeadd_fieldset']['bt_info'] = array(
      '#type' => 'item',
      '#title' => t('No conent types available for Bubbletimer. Set on the <a href="@link">Bubbletimer settings</a> page.', array('@link' => url('admin/settings/bubbletimer/general'))),
    );
  }
  foreach ((array)$nodes_available as $node_type => $nodes) {
    $form['bt_nodeadd_fieldset']['bt_addnode_select_'.$node_type] = array(
      '#type' => 'select',
      '#title' => $node_types_available[$node_type],
      '#options' => $nodes,
      '#attributes' => array(
        'class' => 'bubbletimer_node_select',
        'node_type' => $node_type,
      ),
    );
  }
  $form['bt_nodeadd_fieldset']['bt_addnode_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add'),
  );

  return $form;
}


/**
 * Submit handler for node-adding form
 *
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_node_add_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user(intval($form_state['values']['bt_addnode_select_'.$form_state['values']['bt_node_type']]), TRUE);
}


function theme_bt_wrapper($form) {
  global $base_path;
  $path = $base_path.drupal_get_path('module', 'bubbletimer');

  $nodes_assigned = bubbletimer_get_jobs_assigned(bubbletimer_get_user());
  $nodes_assigned_keys = array_keys($nodes_assigned);

  $table_rows = array();
  $row = array();
  $row[] = array(
    'data' => '<span class="wait bt_chart_updater"></span>Projects <small>(<span class="bt_sum_hour"></span>h a day)</small>',
    'class' => 'bt_td_header',
  );

  $form_keys = array_keys($form['slot']);
  $nids = array_filter($form_keys, create_function('$item', 'return is_numeric($item);'));

  // Left arrow
  $row[] = array(
    'data' => '<img src="'.$path.'/images/step_left.gif" />',
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_left',
  );

  for ($hour = BT_START_HOUR; $hour < BT_END_HOUR; $hour++) {
    $row[] = array(
      'data' => $hour.'h',
      'colspan' => BT_SLOTS_PER_HOUR,
      'hour' => $hour,
      'class' => 'bt_td_header bt_hour_td '.($hour & 1 ? 'even' : 'odd'),
      'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
    );
  }

  // Right arrow
  $row[] = array(
    'data' => '<img src="'.$path.'/images/step_right.gif" />',
    'rowspan' => count($nids) + 1,
    'id' => 'bt_table_step_right',
  );
  $table_rows[] = $row;

  foreach ((array)$form['slot'] as $nid => $form_row) {
    if (!is_array($form_row) || !is_numeric($nid)) continue;
    $row = array();

    $title = array_shift($form['title']);
    //var_dump($title);
    $assign_link = in_array($nid, $nodes_assigned_keys) ?
      '<a class="bubbletimer_node_remove" href="'.url(BT_URL_NODE_REMOVE.'/'.$nid).'">&nbsp;</a>' :
      '<a class="bubbletimer_node_add"    href="'.url(BT_URL_NODE_ADD.'/'.$nid).'">&nbsp;</a>';
    $row[] = array(
      'data' => $assign_link.drupal_render($title),
      'class' => 'bt_title_td',
      'nid' => $nid,
    );

    $hour = BT_START_HOUR;
    foreach ((array)$form_row as $timestamp => $element) {
      //var_dump($element); exit;
      if (!is_numeric($timestamp)) continue;
      //var_dump($timestamp);
      $row[] = array(
        'data' => drupal_render($element),
        'hour' => floor($hour),
        'class' => 'bt_slot_td '.(floor($hour) & 1 ? 'even' : 'odd'),
        'valign' => 'bottom',
        'nid' => $nid,
        'style' => $hour < 8 || $hour >= 18 ? 'display:none;' : '',
      );
      $hour += 1 / BT_SLOTS_PER_HOUR;
    }
    $table_rows[] = $row;
  }

  $output  = theme_table(NULL, $table_rows, array(
    'id' => 'bt_table',
  ));
  //$output .= drupal_render($form);

  return $output;
}


/**
 * Node assign form.
 * @param array $form_original
 * @param int $nid
 * @param string $action
 * @return array
 */
function bubbletimer_assign_node_form($form_original, $nid, $action = BT_NODE_REMOVE) {
  drupal_set_title(t('Bubbletimer - @action node', array('@action' => $action == BT_NODE_ADD ? t('add') : t('remove'))));
  $node = node_load($nid);

 return array(
    'text' => array(
      '#type' => 'item',
      '#title' => t('Do you want to @action the node: @title?', array(
        '@action' => $action == BT_NODE_ADD ? t('add') : t('remove'),
        '@title' => $node->title,
      )),
    ),
    'nid' => array(
      '#type' => 'hidden',
      '#value' => (int)$nid,
    ),
    'action' => array(
      '#type' => 'hidden',
      '#value' => $action,
    ),
    'submit' => array(
      '#type' => 'submit',
      '#value' => $action == BT_NODE_ADD ? t('Add node') : t('Remove node'),
    ),
  );
}


/**
 * Submission for node assign form.
 * @param array $form
 * @param array $form_state
 */
function bubbletimer_assign_node_form_submit($form, $form_state) {
  bubbletimer_assign_node_to_user($form_state['values']['nid'], $form_state['values']['action'] == BT_NODE_ADD);
  drupal_set_message(t('Node assign is saved.'));
  drupal_goto('bubbletimer');
}
